@@ -7,7 +7,7 @@
  *      implementation of the pqxx::connection_base abstract base class.
  *   pqxx::connection_base encapsulates a frontend to backend connection
  *
- * Copyright (c) 2001-2013, Jeroen T. Vermeulen <jtv@xs4all.nl>
+ * Copyright (c) 2001-2011, Jeroen T. Vermeulen <jtv@xs4all.nl>
  *
  * See COPYING for copyright license.  If you did not receive a file called
  * COPYING with this source code, please notify the distributor of this mistake,
@@ -43,6 +43,10 @@
 
 #include "libpq-fe.h"
 
+#ifdef PQXX_QUIET_DESTRUCTORS
+#include "pqxx/errorhandler"
+#endif
+
 #include "pqxx/binarystring"
 #include "pqxx/connection"
 #include "pqxx/connection_base"
@@ -51,11 +55,12 @@
 #include "pqxx/result"
 #include "pqxx/strconv"
 #include "pqxx/transaction"
-#include "pqxx/notify-listen"
+#include "pqxx/notification"
 
+#include "pqxx/internal/gates/connection-reactivation_avoidance_exemption.hxx"
+#include "pqxx/internal/gates/errorhandler-connection.hxx"
 #include "pqxx/internal/gates/result-creation.hxx"
 #include "pqxx/internal/gates/result-connection.hxx"
-#include "pqxx/internal/gates/connection-reactivation_avoidance_exemption.hxx"
 
 using namespace PGSTD;
 using namespace pqxx;
@@ -63,16 +68,6 @@ using namespace pqxx::internal;
 using namespace pqxx::prepare;
 
 
-extern "C"
-{
-// Pass C-linkage notice processor call on to C++-linkage noticer object.  The
-// void * argument points to the noticer.
-static void pqxxNoticeCaller(void *arg, const char *Msg)
-{
-  if (arg && Msg) (*static_cast<pqxx::noticer *>(arg))(Msg);
-}
-
-
 #ifdef PQXX_SELECT_ACCEPTS_NULL
   // The always-empty fd_set
 static fd_set *const fdset_none = 0;
@@ -109,6 +104,15 @@ static void clear_fdmask(fd_set *mask)
 }
 #endif
 
+
+extern "C"
+{
+// The PQnoticeProcessor that receives an error or warning from libpq and sends
+// it to the appropriate connection for processing.
+static void pqxx_notice_processor(void *conn, const char *msg)
+{
+  reinterpret_cast<pqxx::connection_base *>(conn)->process_notice(msg);
+}
 }
 
 
@@ -125,15 +129,15 @@ pqxx::connection_base::connection_base(connectionpolicy &pol) :
   m_Conn(0),
   m_policy(pol),
   m_Trans(),
-  m_Noticer(),
-  m_defaultNoticeProcessor(0),
+  m_errorhandlers(),
   m_Trace(0),
   m_serverversion(0),
   m_reactivation_avoidance(),
   m_unique_id(0),
   m_Completed(false),
   m_inhibit_reactivation(false),
-  m_caps()
+  m_caps(),
+  m_verbosity(normal)
 {
   clearcaps();
 }
@@ -166,7 +170,7 @@ int pqxx::connection_base::backendpid() const throw ()
 
 namespace
 {
-int socket_of(const ::pqxx::internal::pq::PGconn *c)
+int PQXX_PURE socket_of(const ::pqxx::internal::pq::PGconn *c) throw ()
 {
   return c ? PQsocket(c) : -1;
 }
@@ -248,11 +252,7 @@ void pqxx::connection_base::simulate_failure()
 
 int pqxx::connection_base::protocol_version() const throw ()
 {
-#ifdef PQXX_HAVE_PQPROTOCOLVERSION
   return m_Conn ? PQprotocolVersion(m_Conn) : 0;
-#else
-  return 0;
-#endif
 }
 
 
@@ -288,8 +288,8 @@ string pqxx::connection_base::get_variable(const PGSTD::string &Var)
 string pqxx::connection_base::RawGetVar(const PGSTD::string &Var)
 {
   // Is this variable in our local map of set variables?
-  // TODO: Is it safe to read-allocate variables in the "cache?"
-  map<string,string>::const_iterator i=m_Vars.find(Var);
+  // TODO: Could we safely read-allocate variables into m_Vars?
+  map<string,string>::const_iterator i = m_Vars.find(Var);
   if (i != m_Vars.end()) return i->second;
 
   return Exec(("SHOW " + Var).c_str(), 0).at(0).at(0).as(string());
@@ -324,26 +324,25 @@ void pqxx::connection_base::SetupState()
   for (PSMap::iterator p = m_prepared.begin(); p != prepared_end; ++p)
     p->second.registered = false;
 
-  m_defaultNoticeProcessor = 0;
-  if (m_Noticer.get()) switchnoticer(m_Noticer);
+  PQsetNoticeProcessor(m_Conn, pqxx_notice_processor, this);
 
   InternalSetTrace();
 
-  if (!m_listeners.empty() || !m_Vars.empty())
+  if (!m_receivers.empty() || !m_Vars.empty())
   {
     stringstream restore_query;
 
-    // Pipeline all queries needed to restore listeners and variables, so we can
+    // Pipeline all queries needed to restore receivers and variables, so we can
     // send them over in one go.
 
-    // Reinstate all active listeners
-    if (!m_listeners.empty())
+    // Reinstate all active receivers
+    if (!m_receivers.empty())
     {
-      const listenerlist::const_iterator End = m_listeners.end();
+      const receiver_list::const_iterator End = m_receivers.end();
       string Last;
-      for (listenerlist::const_iterator i = m_listeners.begin(); i != End; ++i)
+      for (receiver_list::const_iterator i = m_receivers.begin(); i != End; ++i)
       {
-        // m_listeners can handle multiple listeners waiting on the same event;
+        // m_receivers can handle multiple receivers waiting on the same event;
         // issue just one LISTEN for each event.
         if (i->first != Last)
         {
@@ -474,92 +473,60 @@ bool pqxx::connection_base::is_open() const throw ()
 }
 
 
-void pqxx::connection_base::switchnoticer(const PGSTD::auto_ptr<noticer> &N)
-	throw ()
-{
-  const PQnoticeProcessor old =
-	PQsetNoticeProcessor(m_Conn, pqxxNoticeCaller, N.get());
-  if (!m_defaultNoticeProcessor)
-    m_defaultNoticeProcessor = old;
-}
-
-
-auto_ptr<pqxx::noticer>
-pqxx::connection_base::set_noticer(PGSTD::auto_ptr<noticer> N) throw ()
-{
-  if (m_Conn)
-  {
-    if (N.get()) switchnoticer(N);
-    else PQsetNoticeProcessor(m_Conn, m_defaultNoticeProcessor, 0);
-  }
-
-  auto_ptr<noticer> Old = m_Noticer;
-  m_Noticer = N;
-
-  return Old;
-}
-
-
 void pqxx::connection_base::process_notice_raw(const char msg[]) throw ()
 {
-  if (msg && *msg)
-  {
-    // TODO: Read default noticer on startup!
-    if (m_Noticer.get()) (*m_Noticer.get())(msg);
-    else fputs(msg, stderr);
-  }
+  if (!msg || !*msg) return;
+  const list<errorhandler *>::const_reverse_iterator
+	rbegin = m_errorhandlers.rbegin(),
+	rend = m_errorhandlers.rend();
+  for (
+	list<errorhandler *>::const_reverse_iterator i = rbegin;
+	i != rend && (**i)(msg);
+	++i) ;
 }
 
 
 void pqxx::connection_base::process_notice(const char msg[]) throw ()
 {
-  if (!msg)
+  if (!msg) return;
+  const size_t len = strlen(msg);
+  if (len == 0) return;
+  if (msg[len-1] == '\n')
   {
-    process_notice_raw("NULL pointer in client program message!\n");
+    process_notice_raw(msg);
   }
-  else
+  else try
   {
-    const size_t len = strlen(msg);
-    if (len > 0)
+    // Newline is missing.  Try the C++ string version of this function.
+    process_notice(string(msg));
+  }
+  catch (const exception &)
+  {
+    // If we can't even do that, use plain old buffer copying instead
+    // (unavoidably, this will break up overly long messages!)
+    const char separator[] = "[...]\n";
+    char buf[1007];
+    size_t bytes = sizeof(buf)-sizeof(separator)-1;
+    size_t written;
+    strcpy(&buf[bytes], separator);
+    // Write all chunks but last.  Each will fill the buffer exactly.
+    for (written = 0; (written+bytes) < len; written += bytes)
     {
-      if (msg[len-1] == '\n')
-      {
-        process_notice_raw(msg);
-      }
-      else try
-      {
-        // Newline is missing.  Try the C++ string version of this function.
-        process_notice(string(msg));
-      }
-      catch (const exception &)
-      {
-        // If we can't even do that, use plain old buffer copying instead
-        // (unavoidably, this will break up overly long messages!)
-        const char separator[] = "[...]\n";
-        char buf[1007];
-        size_t bytes = sizeof(buf)-sizeof(separator)-1;
-        size_t written;
-        strcpy(&buf[bytes], separator);
-        // Write all chunks but last.  Each will fill the buffer exactly.
-        for (written = 0; (written+bytes) < len; written += bytes)
-        {
-          memcpy(buf, &msg[written], bytes);
-          process_notice_raw(buf);
-        }
-        // Write any remaining bytes (which won't fill an entire buffer)
-        bytes = len-written;
-        memcpy(buf, &msg[written], bytes);
-        // Add trailing nul byte, plus newline unless there already is one
-        strcpy(&buf[bytes], &"\n"[buf[bytes-1]=='\n']);
-        process_notice_raw(buf);
-      }
+      memcpy(buf, &msg[written], bytes);
+      process_notice_raw(buf);
     }
+    // Write any remaining bytes (which won't fill an entire buffer)
+    bytes = len-written;
+    memcpy(buf, &msg[written], bytes);
+    // Add trailing nul byte, plus newline unless there already is one
+    strcpy(&buf[bytes], &"\n"[buf[bytes-1]=='\n']);
+    process_notice_raw(buf);
   }
 }
 
 void pqxx::connection_base::process_notice(const PGSTD::string &msg) throw ()
 {
-  // Ensure that message passed to noticer ends in newline
+  // Ensure that message passed to errorhandler ends in newline
   if (msg[msg.size()-1] == '\n')
   {
     process_notice_raw(msg.c_str());
@@ -586,18 +553,18 @@ void pqxx::connection_base::trace(FILE *Out) throw ()
 }
 
 
-void pqxx::connection_base::add_listener(pqxx::notify_listener *T)
+void pqxx::connection_base::add_receiver(pqxx::notification_receiver *T)
 {
-  if (!T) throw argument_error("Null listener registered");
+  if (!T) throw argument_error("Null receiver registered");
 
-  // Add to listener list and attempt to start listening.
-  const listenerlist::iterator p = m_listeners.find(T->name());
-  const listenerlist::value_type NewVal(T->name(), T);
+  // Add to receiver list and attempt to start listening.
+  const receiver_list::iterator p = m_receivers.find(T->channel());
+  const receiver_list::value_type NewVal(T->channel(), T);
 
-  if (p == m_listeners.end())
+  if (p == m_receivers.end())
   {
     // Not listening on this event yet, start doing so.
-    const string LQ("LISTEN \"" + T->name() + "\"");
+    const string LQ("LISTEN \"" + T->channel() + "\"");
 
     if (is_open()) try
     {
@@ -606,41 +573,42 @@ void pqxx::connection_base::add_listener(pqxx::notify_listener *T)
     catch (const broken_connection &)
     {
     }
-    m_listeners.insert(NewVal);
+    m_receivers.insert(NewVal);
   }
   else
   {
-    m_listeners.insert(p, NewVal);
+    m_receivers.insert(p, NewVal);
   }
 }
 
 
-void pqxx::connection_base::remove_listener(pqxx::notify_listener *T) throw ()
+void pqxx::connection_base::remove_receiver(pqxx::notification_receiver *T)
+	throw ()
 {
   if (!T) return;
 
   try
   {
     // Keep Sun compiler happy...  Hope it doesn't annoy other compilers
-    pair<const string, notify_listener *> tmp_pair(T->name(), T);
-    listenerlist::value_type E = tmp_pair;
+    pair<const string, notification_receiver *> tmp_pair(T->channel(), T);
+    receiver_list::value_type E = tmp_pair;
 
-    typedef pair<listenerlist::iterator, listenerlist::iterator> Range;
-    Range R = m_listeners.equal_range(E.first);
+    typedef pair<receiver_list::iterator, receiver_list::iterator> Range;
+    Range R = m_receivers.equal_range(E.first);
 
-    const listenerlist::iterator i = find(R.first, R.second, E);
+    const receiver_list::iterator i = find(R.first, R.second, E);
 
     if (i == R.second)
     {
-      process_notice("Attempt to remove unknown listener '" + E.first + "'");
+      process_notice("Attempt to remove unknown receiver '" + E.first + "'");
     }
     else
     {
-      // Erase first; otherwise a notification for the same listener may yet
+      // Erase first; otherwise a notification for the same receiver may yet
       // come in and wreak havoc.  Thanks Dragan Milenkovic.
       const bool gone = (m_Conn && (R.second == ++R.first));
-      m_listeners.erase(i);
-      if (gone) Exec(("UNLISTEN \"" + T->name() + "\"").c_str(), 0);
+      m_receivers.erase(i);
+      if (gone) Exec(("UNLISTEN \"" + T->channel() + "\"").c_str(), 0);
     }
   }
   catch (const exception &e)
@@ -714,50 +682,43 @@ void pqxx::connection_base::cancel_query()
   cancel();
 }
 
-
-namespace
+void pqxx::connection_base::set_verbosity(error_verbosity verbosity) throw ()
 {
-void freemem_notif(PGnotify *p) throw ()
-{
-#ifdef PQXX_HAVE_PQFREENOTIFY
-  PQfreeNotify(p);
-#else
-  freepqmem(p);
-#endif
+    PQsetErrorVerbosity(m_Conn, static_cast<PGVerbosity>(verbosity));
+    m_verbosity = verbosity;
 }
-} // namespace
 
 
 int pqxx::connection_base::get_notifs()
 {
-  int notifs = 0;
-  if (!is_open()) return notifs;
+  if (!is_open()) return 0;
 
   if (!consume_input()) throw broken_connection();
 
   // Even if somehow we receive notifications during our transaction, don't
   // deliver them.
-  if (m_Trans.get()) return notifs;
+  if (m_Trans.get()) return 0;
 
-  typedef PQAlloc<PGnotify, freemem_notif> notifptr;
+  int notifs = 0;
+  typedef PQAlloc<PGnotify> notifptr;
   for (notifptr N( PQnotifies(m_Conn) );
        N.get();
        N = notifptr(PQnotifies(m_Conn)))
   {
-    typedef listenerlist::iterator TI;
+    typedef receiver_list::iterator TI;
 
     notifs++;
 
-    pair<TI, TI> Hit = m_listeners.equal_range(string(N->relname));
+    pair<TI, TI> Hit = m_receivers.equal_range(string(N->relname));
     for (TI i = Hit.first; i != Hit.second; ++i) try
     {
-      (*i->second)(N->be_pid);
+      (*i->second)(N->extra, N->be_pid);
     }
     catch (const exception &e)
     {
       try
       {
-        process_notice("Exception in notification listener '" +
+        process_notice("Exception in notification receiver '" +
 		       i->first +
 		       "': " +
 		       e.what() +
@@ -766,12 +727,12 @@ int pqxx::connection_base::get_notifs()
       catch (const bad_alloc &)
       {
         // Out of memory.  Try to get the message out in a more robust way.
-        process_notice("Exception in notification listener, "
+        process_notice("Exception in notification receiver, "
 	    "and also ran out of memory\n");
       }
       catch (const exception &)
       {
-        process_notice("Exception in notification listener "
+        process_notice("Exception in notification receiver "
 	    "(compounded by other error)\n");
       }
     }
@@ -816,6 +777,34 @@ const char *pqxx::connection_base::ErrMsg() const throw ()
 }
 
 
+void pqxx::connection_base::register_errorhandler(errorhandler *handler)
+{
+  m_errorhandlers.push_back(handler);
+}
+
+
+void pqxx::connection_base::unregister_errorhandler(errorhandler *handler)
+  throw ()
+{
+  // The errorhandler itself will take care of nulling its pointer to this
+  // connection.
+  m_errorhandlers.remove(handler);
+}
+
+
+vector<errorhandler *> pqxx::connection_base::get_errorhandlers() const
+{
+  vector<errorhandler *> handlers;
+  handlers.reserve(m_errorhandlers.size());
+  for (
+	list<errorhandler *>::const_iterator i = m_errorhandlers.begin(); 
+	i != m_errorhandlers.end();
+	++i)
+    handlers.push_back(*i);
+  return handlers;
+}
+
+
 pqxx::result pqxx::connection_base::Exec(const char Query[], int Retries)
 {
   activate();
@@ -836,7 +825,7 @@ pqxx::result pqxx::connection_base::Exec(const char Query[], int Retries)
 }
 
 
-pqxx::prepare::declaration pqxx::connection_base::prepare(
+void pqxx::connection_base::prepare(
 	const PGSTD::string &name,
 	const PGSTD::string &definition)
 {
@@ -857,25 +846,19 @@ pqxx::prepare::declaration pqxx::connection_base::prepare(
       i->second.registered = false;
       i->second.definition = definition;
     }
-
-    // Prepare for new definition of parameters
-    i->second.parameters.clear();
-    i->second.varargs = false;
-    i->second.complete = false;
   }
   else
   {
-    m_prepared.insert(make_pair(name,
-	  prepare::internal::prepared_def(definition)));
+    m_prepared.insert(make_pair(
+	name,
+	prepare::internal::prepared_def(definition)));
   }
-  return prepare::declaration(*this, name);
 }
 
 
-pqxx::prepare::declaration pqxx::connection_base::prepare(
-	const PGSTD::string &definition)
+void pqxx::connection_base::prepare(const PGSTD::string &definition)
 {
-  return this->prepare(string(), definition);
+  this->prepare(string(), definition);
 }
 
 
@@ -892,54 +875,6 @@ void pqxx::connection_base::unprepare(const PGSTD::string &name)
 }
 
 
-namespace
-{
-string escape_param(connection_base &C,
-	const char in[],
-	int len,
-	prepare::param_treatment treatment)
-{
-  if (!in) return "null";
-
-  switch (treatment)
-  {
-  case treat_binary:
-    return "'" + escape_binary(string(in, string::size_type(len))) + "'";
-
-  case treat_string:
-    return C.quote(in);
-
-  case treat_bool:
-    switch (in[0])
-    {
-    case 't':
-    case 'T':
-    case 'f':
-    case 'F':
-      break;
-    default:
-      {
-        // Looks like a numeric value may have been passed.  Try to convert it
-        // back to a number, then to bool to normalize its representation.
-        bool b;
-	from_string(in,b);
-        return to_string(b);
-      }
-    }
-    break;
-
-  case treat_direct:
-    break;
-
-  default:
-    throw usage_error("Unknown treatment for prepared-statement parameter");
-  }
-
-  return in;
-}
-} // namespace
-
-
 pqxx::prepare::internal::prepared_def &
 pqxx::connection_base::find_prepared(const PGSTD::string &statement)
 {
@@ -949,78 +884,26 @@ pqxx::connection_base::find_prepared(const PGSTD::string &statement)
   return s->second;
 }
 
-void pqxx::connection_base::prepare_param_declare(
-	const PGSTD::string &statement,
-	const PGSTD::string &sqltype,
-	param_treatment treatment)
-{
-  prepare::internal::prepared_def &s = find_prepared(statement);
-  if (s.complete)
-    throw usage_error("Attempt to add parameter to prepared statement " +
-	statement +
-	" after its definition was completed");
-  if (s.varargs)
-    throw usage_error("Attempt to add parameters to prepared statement " +
-	statement + " after arbitrary trailing parameters.");
-  s.addparam(sqltype,treatment);
-}
-
-
-void pqxx::connection_base::prepare_param_declare_varargs(
-	const PGSTD::string &statement,
-	param_treatment treatment)
-{
-  if (!supports(cap_statement_varargs))
-    throw feature_not_supported(
-	"Prepared statements do not support variable argument lists "
-	"in this configuration.");
-  prepare::internal::prepared_def &s = find_prepared(statement);
-  if (s.complete)
-    throw usage_error("Attempt to add arbitrary parameters to prepared "
-	"statement " + statement + " after its definition was completed.");
-  s.varargs_treatment = treatment;
-  s.varargs = true;
-  s.complete = true;
-}
- 
 
 pqxx::prepare::internal::prepared_def &
 pqxx::connection_base::register_prepared(const PGSTD::string &name)
 {
   activate();
+  if (!supports(cap_prepared_statements) || protocol_version() < 3)
+    throw feature_not_supported(
+	"Prepared statements in libpqxx require a newer server version.");
 
   prepare::internal::prepared_def &s = find_prepared(name);
 
-  s.complete = true;
-
   // "Register" (i.e., define) prepared statement with backend on demand
-  if (!s.registered && supports(cap_prepared_statements))
+  if (!s.registered)
   {
-#ifdef PQXX_HAVE_PQPREPARE
-    if (protocol_version() >= 3)
-    {
-      result r = make_result(
-	PQprepare(m_Conn, name.c_str(), s.definition.c_str(), 0, 0),
-	"[PREPARE " + name + "]");
-      check_result(r);
-      s.registered = !name.empty();
-      return s;
-    }
-#endif
-    stringstream P;
-    P << "PREPARE \"" << name << "\" ";
-
-    if (!s.parameters.empty())
-      P << '('
-	<< separated_list(",",
-		s.parameters.begin(),
-		s.parameters.end(),
-		prepare::internal::get_sqltype())
-	<< ')';
-
-    P << " AS " << s.definition;
-    Exec(P.str().c_str(), 0);
+    result r = make_result(
+      PQprepare(m_Conn, name.c_str(), s.definition.c_str(), 0, 0),
+      "[PREPARE " + name + "]");
+    check_result(r);
     s.registered = !name.empty();
+    return s;
   }
 
   return s;
@@ -1037,91 +920,22 @@ pqxx::result pqxx::connection_base::prepared_exec(
 	const PGSTD::string &statement,
 	const char *const params[],
 	const int paramlengths[],
+	const int binary[],
 	int nparams)
 {
-  prepare::internal::prepared_def &s = register_prepared(statement);
-
-  const int expected_params = int(s.parameters.size());
-  if (nparams < expected_params)
-    throw usage_error("Insufficient parameters for prepared statement " +
-	statement + ": expected " + to_string(expected_params) + ", "
-	"received " + to_string(nparams));
-
-  if (nparams > expected_params && !s.varargs)
-    throw usage_error("Too many arguments for prepared statement " +
-	statement + ": expected " + to_string(expected_params) + ", "
-	"received " + to_string(nparams));
-
-  result r;
-
+  register_prepared(statement);
   activate();
-
-  if (supports(cap_prepared_statements))
-  {
-#ifdef PQXX_HAVE_PQEXECPREPARED
-    if (protocol_version() >= 3)
-    {
-      internal::scoped_array<int> binary(size_t(nparams+1));
-      for (int i=0; i<expected_params; ++i)
-        binary[i] = (s.parameters[size_t(i)].treatment == treat_binary);
-      for (int j=expected_params; j < nparams; ++j)
-        binary[j] = (s.varargs_treatment == treat_binary);
-      binary[nparams] = 0;
-
-      r = make_result(PQexecPrepared(m_Conn,
+  result r = make_result(
+	PQexecPrepared(
+		m_Conn,
 		statement.c_str(),
 		nparams,
 		params,
 		paramlengths,
-		binary.get(),
+		binary,
 		0),
-	statement);
-
-      check_result(r);
-      get_notifs();
-      return r;
-    }
-#endif
-    stringstream Q;
-    Q << "EXECUTE \"" << statement << '"';
-    if (nparams)
-    {
-      Q << " (";
-      for (int a = 0; a < nparams; ++a)
-      {
-	Q << escape_param(*this,
-		params[a],
-		paramlengths[a],
-		(a < expected_params) ?
-			s.parameters[size_t(a)].treatment :
-			s.varargs_treatment);
-	if (a < nparams-1) Q << ',';
-      }
-      Q << ')';
-    }
-    r = Exec(Q.str().c_str(), 0);
-  }
-  else
-  {
-    stringstream Q;
-    // This backend doesn't support prepared statements.  Do our own variable
-    // substitution.
-    string S = s.definition;
-    for (int n = nparams-1; n >= 0; --n)
-    {
-      const string key = "$" + to_string(n+1),
-	           val = escape_param(*this,
-				params[n],
-				paramlengths[n],
-				s.parameters[size_t(n)].treatment);
-      const string::size_type keysz = key.size();
-      // TODO: Skip quoted strings!  (And careful with multibyte encodings...)
-      for (string::size_type h=S.find(key); h!=string::npos; h=S.find(key))
-	S.replace(h,keysz,val);
-    }
-    Q << S;
-    r = Exec(Q.str().c_str(), 0);
-  }
+    	statement);
+  check_result(r);
   get_notifs();
   return r;
 }
@@ -1164,8 +978,7 @@ void pqxx::connection_base::close() throw ()
 {
   m_Completed = false;
 #ifdef PQXX_QUIET_DESTRUCTORS
-  auto_ptr<noticer> n(new nonnoticer());
-  set_noticer(n);
+  quiet_errorhandler quiet(*this);
 #endif
   inhibit_reactivation(false);
   m_reactivation_avoidance.clear();
@@ -1175,12 +988,21 @@ void pqxx::connection_base::close() throw ()
       process_notice("Closing connection while " +
 	             m_Trans.get()->description() + " still open");
 
-    if (!m_listeners.empty())
+    if (!m_receivers.empty())
     {
-      process_notice("Closing connection with outstanding listeners");
-      m_listeners.clear();
+      process_notice("Closing connection with outstanding receivers.");
+      m_receivers.clear();
     }
 
+    PQsetNoticeProcessor(m_Conn, NULL, NULL);
+    list<errorhandler *> old_handlers;
+    m_errorhandlers.swap(old_handlers);
+    const list<errorhandler *>::const_reverse_iterator
+	rbegin = old_handlers.rbegin(),
+	rend = old_handlers.rend();
+    for (list<errorhandler *>::const_reverse_iterator i = rbegin; i!=rend; ++i)
+      gate::errorhandler_connection_base(**i).unregister();
+
     m_Conn = m_policy.do_disconnect(m_Conn);
   }
   catch (...)
@@ -1240,14 +1062,6 @@ void pqxx::connection_base::UnregisterTransaction(transaction_base *T)
 }
 
 
-#ifndef PQXX_HAVE_PQPUTCOPY
-namespace
-{
-const string theWriteTerminator = "\\.";
-}
-#endif
-
-
 bool pqxx::connection_base::ReadCopyLine(PGSTD::string &Line)
 {
   if (!is_open())
@@ -1256,7 +1070,6 @@ bool pqxx::connection_base::ReadCopyLine(PGSTD::string &Line)
   Line.erase();
   bool Result;
 
-#ifdef PQXX_HAVE_PQPUTCOPY
   char *Buf = 0;
   const string query = "[END COPY]";
   switch (PQgetCopyData(m_Conn, &Buf, false))
@@ -1283,45 +1096,6 @@ bool pqxx::connection_base::ReadCopyLine(PGSTD::string &Line)
       }
       Result = true;
   }
-#else
-  char Buf[10000];
-  bool LineComplete = false;
-
-  while (!LineComplete)
-  {
-    switch (PQgetline(m_Conn, Buf, sizeof(Buf)))
-    {
-    case EOF:
-      PQendcopy(m_Conn);
-      throw failure("Unexpected EOF from backend");
-
-    case 0:
-      LineComplete = true;
-      break;
-
-    case 1:
-      break;
-
-    default:
-      throw failure("Unexpected COPY response from backend");
-    }
-
-    Line += Buf;
-  }
-
-  Result = (Line != theWriteTerminator);
-
-  if (Result)
-  {
-    if (!Line.empty() && (Line[Line.size()-1] == '\n'))
-      Line.erase(Line.size()-1);
-  }
-  else
-  {
-    Line.erase();
-    if (PQendcopy(m_Conn)) throw failure(ErrMsg());
-  }
-#endif
 
   return Result;
 }
@@ -1336,23 +1110,17 @@ void pqxx::connection_base::WriteCopyLine(const PGSTD::string &Line)
   const char *const LC = L.c_str();
   const string::size_type Len = L.size();
 
-#ifdef PQXX_HAVE_PQPUTCOPY
   if (PQputCopyData(m_Conn, LC, int(Len)) <= 0)
   {
     const string Msg = string("Error writing to table: ") + ErrMsg();
     PQendcopy(m_Conn);
     throw failure(Msg);
   }
-#else
-  if (PQputnbytes(m_Conn, LC, int(Len)) == EOF)
-    throw failure(string("Error writing to table: ") + ErrMsg());
-#endif
 }
 
 
 void pqxx::connection_base::EndCopyWrite()
 {
-#ifdef PQXX_HAVE_PQPUTCOPY
   int Res = PQputCopyEnd(m_Conn, NULL);
   switch (Res)
   {
@@ -1370,15 +1138,6 @@ void pqxx::connection_base::EndCopyWrite()
   }
 
   check_result(make_result(PQgetResult(m_Conn), "[END COPY]"));
-
-#else
-  WriteCopyLine(theWriteTerminator);
-  // This check is a little odd, but for some reason PostgreSQL 7.4 keeps
-  // returning 1 (i.e., failure) but with an empty error message, and without
-  // anything seeming wrong.
-  if ((PQendcopy(m_Conn) != 0) && ErrMsg() && *ErrMsg())
-    throw failure(ErrMsg());
-#endif
 }
 
 
@@ -1406,7 +1165,6 @@ string pqxx::connection_base::esc(const char str[], size_t maxlen)
 {
   string escaped;
 
-#if defined(PQXX_HAVE_PQESCAPESTRINGCONN)
   // We need a connection object...  This is the one reason why this function is
   // not const!
   if (!m_Conn) activate();
@@ -1426,39 +1184,6 @@ string pqxx::connection_base::esc(const char str[], size_t maxlen)
   }
   delete [] buf;
 
-#elif defined(PQXX_HAVE_PQESCAPESTRING)
-  scoped_array<char> buf(new char[2*maxlen+1]);
-  const size_t bytes = PQescapeString(buf.get(), str, maxlen);
-  escaped.assign(buf.get(), bytes);
-
-#else
-  // Last-ditch workaround.  This has serious problems with multibyte encodings.
-  for (size_t i=0; str[i] && (i < maxlen); ++i)
-  {
-    // Ensure we don't pass negative integers to isprint()/isspace(), which
-    // Visual C++ chokes on.
-    const unsigned char c(str[i]);
-    if (c & 0x80)
-    {
-      throw failure("non-ASCII text passed to sqlesc(); "
-	  "the libpq version that libpqxx was built with does not support this "
-	  "yet (minimum is postgres 7.2)");
-    }
-    else if (isprint(c) || isspace(c))
-    {
-      if (c=='\\' || c=='\'') escaped += c;
-      escaped += c;
-    }
-    else
-    {
-        char s[8];
-	// TODO: Number may be formatted according to locale!  :-(
-        sprintf(s, "\\%03o", static_cast<unsigned int>(c));
-        escaped.append(s, 4);
-    }
-  }
-#endif
-
   return escaped;
 }
 
@@ -1478,26 +1203,49 @@ string pqxx::connection_base::esc(const PGSTD::string &str)
 string pqxx::connection_base::esc_raw(const unsigned char str[], size_t len)
 {
   size_t bytes = 0;
-#ifdef PQXX_HAVE_PQESCAPEBYTEACONN
   // We need a connection object...  This is the one reason why this function is
   // not const!
-  if (!m_Conn) activate();
+  activate();
 
   PQAlloc<unsigned char> buf( PQescapeByteaConn(m_Conn, str, len, &bytes) );
-  if (!buf.get())
-  {
-    // TODO: Distinguish different errors in exception type
-    throw failure(ErrMsg());
-  }
-#else
-  unsigned char *const s = const_cast<unsigned char *>(str);
-  PQAlloc<unsigned char> buf( PQescapeBytea(s, len, &bytes) );
   if (!buf.get()) throw bad_alloc();
-#endif
   return string(reinterpret_cast<char *>(buf.get()));
 }
 
 
+string pqxx::connection_base::quote_raw(const unsigned char str[], size_t len)
+{
+  return "'" + esc_raw(str, len) + "'::bytea";
+}
+
+
+string pqxx::connection_base::quote(const binarystring &b)
+{
+  return quote_raw(b.data(), b.size());
+}
+
+
+string pqxx::connection_base::quote_name(const string &identifier)
+{
+#ifdef PQXX_HAVE_PQESCAPEIDENTIFIER
+  // We need a connection object...  This is the one reason why this function is
+  // not const!
+  activate();
+  PQAlloc<char> buf(
+	PQescapeIdentifier(m_Conn, identifier.c_str(), identifier.size()));
+  if (!buf.get()) throw failure(ErrMsg());
+  return string(buf.get());
+#else
+  for (string::const_iterator i = identifier.begin(); i!=identifier.end(); ++i)
+    if (*i == '\\' || *i == '\'' || *i == '"' || *i == '\0')
+      throw feature_not_supported(
+	"Identifier contains characters that are difficult to escape; "
+	"this requires a newer libpq version.");
+  return "\"" + identifier + "\"";
+#endif
+}
+
+
 pqxx::internal::reactivation_avoidance_exemption::
   reactivation_avoidance_exemption(
 	connection_base &C) :
@@ -1549,7 +1297,11 @@ void pqxx::internal::wait_read(const pq::PGconn *c,
     long seconds,
     long microseconds)
 {
-  timeval tv = { seconds, microseconds };
+  // These are really supposed to be time_t and suseconds_t.  But not all
+  // platforms have that type; some use "long" instead, and some 64-bit
+  // systems use 32-bit integers here.  So "int" seems to be the only really
+  // safe type to use.
+  timeval tv = { time_t(seconds), int(microseconds) };
   wait_fd(socket_of(c), false, &tv);
 }
 
@@ -1606,46 +1358,15 @@ int pqxx::connection_base::await_notification(long seconds, long microseconds)
 
 void pqxx::connection_base::read_capabilities() throw ()
 {
-#ifdef PQXX_HAVE_PQSERVERVERSION
   m_serverversion = PQserverVersion(m_Conn);
-#else
-  m_serverversion = 0;
-  try
-  {
-    // Estimate server version by querying 'version()'.  This may not be exact!
-    const string VQ = "SELECT version()";
-    const result r = gate::result_creation::create(
-	PQexec(m_Conn, VQ.c_str()),
-	protocol_version(),
-	VQ,
-	encoding_code());
-
-    int x=0, y=0, z=0;
-    if ((sscanf(r[0][0].c_str(), "PostgreSQL %d.%d.%d", &x, &y, &z) == 3) &&
-        (x >= 0) && (x < 100) &&
-	(y >= 0) && (y < 100) &&
-	(z >= 0) && (z < 100))
-      m_serverversion = 10000*x + 100*y + z;
-  }
-  catch (const exception &)
-  {
-  }
-#endif
 
   const int v = m_serverversion;
-#if defined(PQXX_HAVE_PQPREPARE) || defined(PQXX_HAVE_PQFTABLECOL)
   const int p = protocol_version();
-#endif
 
   m_caps[cap_prepared_statements] = (v >= 70300);
 
-#ifdef PQXX_HAVE_PQPREPARE
   m_caps[cap_statement_varargs] = (v >= 70300 && (p >= 3));
   m_caps[cap_prepare_unnamed_statement] = (p >= 3);
-#else
-  m_caps[cap_statement_varargs] = false;
-  m_caps[cap_prepare_unnamed_statement] = false;
-#endif
 
   m_caps[cap_cursor_scroll] = (v >= 70400);
   m_caps[cap_cursor_with_hold] = (v >= 70400);
@@ -1654,13 +1375,11 @@ void pqxx::connection_base::read_capabilities() throw ()
   m_caps[cap_create_table_with_oids] = (v >= 80000);
   m_caps[cap_read_only_transactions] = (v >= 80000);
 
-#ifdef PQXX_HAVE_PQFTABLECOL
+  m_caps[cap_notify_payload] = (v >= 90000);
+
   m_caps[cap_table_column] = (p >= 3);
-#endif
 
-#ifdef PQXX_HAVE_PQEXECPARAMS
   m_caps[cap_parameterized_statements] = (p >= 3);
-#endif
 }
 
 
@@ -1671,9 +1390,9 @@ string pqxx::connection_base::adorn_name(const PGSTD::string &n)
 }
 
 
-int pqxx::connection_base::encoding_code() throw ()
+int pqxx::connection_base::encoding_code()
 {
-#ifdef PQXX_HAVE_PQENCODING
+#ifdef PQXX_HAVE_PQCLIENTENCODING
   activate();
   return PQclientEncoding(m_Conn);
 #else
@@ -1682,11 +1401,11 @@ int pqxx::connection_base::encoding_code() throw ()
 }
 
 
-#ifdef PQXX_HAVE_PQEXECPARAMS
 pqxx::result pqxx::connection_base::parameterized_exec(
 	const PGSTD::string &query,
 	const char *const params[],
 	const int paramlengths[],
+	const int binaries[],
 	int nparams)
 {
   if (!supports(cap_parameterized_statements)) throw feature_not_supported(
@@ -1700,21 +1419,10 @@ pqxx::result pqxx::connection_base::parameterized_exec(
 		NULL,
 		params,
 		paramlengths,
-		NULL,
+		binaries,
 		0),
 	query);
   check_result(r);
   get_notifs();
   return r;
 }
-#else
-pqxx::result pqxx::connection_base::parameterized_exec(
-	const PGSTD::string &,
-	const char *const[],
-	const int[],
-	int)
-{
-  throw feature_not_supported(
-	"This libpq version does not support parameterized statements.");
-}
-#endif // PQXX_HAVE_PQEXECPARAMS
