@@ -7,7 +7,7 @@
  *      Implementation of the Large Objects interface
  *   Allows access to large objects directly, or though I/O streams
  *
- * Copyright (c) 2003-2013, Jeroen T. Vermeulen <jtv@xs4all.nl>
+ * Copyright (c) 2003-2011, Jeroen T. Vermeulen <jtv@xs4all.nl>
  *
  * See COPYING for copyright license.  If you did not receive a file called
  * COPYING with this source code, please notify the distributor of this mistake,
@@ -221,9 +221,10 @@ pqxx::largeobjectaccess::cseek(off_type dest, seekdir dir) throw ()
 pqxx::largeobjectaccess::pos_type
 pqxx::largeobjectaccess::cwrite(const char Buf[], size_type Len) throw ()
 {
-  const pos_type bytes_written = lo_write(
-	RawConnection(), m_fd, const_cast<char *>(Buf), size_t(Len));
-  return PGSTD::max(bytes_written, -1L);
+  return
+    PGSTD::max(
+	lo_write(RawConnection(), m_fd,const_cast<char *>(Buf), size_t(Len)),
+        -1);
 }
 
 
@@ -298,7 +299,7 @@ void pqxx::largeobjectaccess::open(openmode mode)
 void pqxx::largeobjectaccess::close() throw ()
 {
 #ifdef PQXX_QUIET_DESTRUCTORS
-  disable_noticer Quiet(m_Trans.conn());
+  quiet_errorhandler quiet(m_Trans.conn());
 #endif
   if (m_fd >= 0) lo_close(RawConnection(), m_fd);
 }
