@@ -5,9 +5,9 @@
  *
  *   DESCRIPTION
  *      implementation of the pqxx::notify_listener class.
- *   pqxx::notify_listener describes a notification to wait on, and what it does
+ *   Predecessor to notification_receiver.  Deprecated.  Do not use.
  *
- * Copyright (c) 2009, Jeroen T. Vermeulen <jtv@xs4all.nl>
+ * Copyright (c) 2009-2011, Jeroen T. Vermeulen <jtv@xs4all.nl>
  *
  * See COPYING for copyright license.  If you did not receive a file called
  * COPYING with this source code, please notify the distributor of this mistake,
@@ -17,29 +17,29 @@
  */
 #include "pqxx/compiler-internal.hxx"
 
+#include <functional>
 #include <string>
 
 #include "pqxx/notify-listen"
 
-#include "pqxx/internal/gates/connection-notify-listener.hxx"
 
-using namespace pqxx::internal;
+void pqxx::internal::notify_listener_forwarder::operator()(
+	const PGSTD::string &,
+	int backend_pid)
+{
+  (*m_wrappee)(backend_pid);
+}
+
 
 pqxx::notify_listener::notify_listener(
 	connection_base &c,
 	const PGSTD::string &n) :
   m_conn(c),
-  m_Name(n)
+  m_forwarder(c, n, this)
 {
-  gate::connection_notify_listener(c).add_listener(this);
 }
 
 
 pqxx::notify_listener::~notify_listener() throw ()
 {
-  connection_base &c = conn();
-#ifdef PQXX_QUIET_DESTRUCTORS
-  disable_noticer Quiet(c);
-#endif
-  gate::connection_notify_listener(c).remove_listener(this);
 }
