@@ -7,7 +7,7 @@
  *      implementation of the pqxx::result class and support classes.
  *   pqxx::result represents the set of result tuples from a database query
  *
- * Copyright (c) 2001-2010, Jeroen T. Vermeulen <jtv@xs4all.nl>
+ * Copyright (c) 2001-2011, Jeroen T. Vermeulen <jtv@xs4all.nl>
  *
  * See COPYING for copyright license.  If you did not receive a file called
  * COPYING with this source code, please notify the distributor of this mistake,
@@ -24,7 +24,9 @@
 #include "libpq-fe.h"
 
 #include "pqxx/except"
+#include "pqxx/field"
 #include "pqxx/result"
+#include "pqxx/tuple"
 
 
 using namespace PGSTD;
@@ -76,43 +78,21 @@ bool pqxx::result::operator==(const result &rhs) const throw ()
 }
 
 
-bool pqxx::result::tuple::operator==(const tuple &rhs) const throw ()
+pqxx::result::const_reverse_iterator pqxx::result::rbegin() const
 {
-  if (&rhs == this) return true;
-  const size_type s(size());
-  if (rhs.size() != s) return false;
-  // TODO: Depends on how null is handled!
-  for (size_type i=0; i<s; ++i) if ((*this)[i] != rhs[i]) return false;
-  return true;
+  return const_reverse_iterator(end());
 }
 
 
-void pqxx::result::tuple::swap(tuple &rhs) throw ()
+pqxx::result::const_reverse_iterator pqxx::result::rend() const
 {
-  const result *const h(m_Home);
-  const result::size_type i(m_Index);
-  const size_type b(m_Begin);
-  const size_type e(m_End);
-  m_Home = rhs.m_Home;
-  m_Index = rhs.m_Index;
-  m_Begin = rhs.m_Begin;
-  m_End = rhs.m_End;
-  rhs.m_Home = h;
-  rhs.m_Index = i;
-  rhs.m_Begin = b;
-  rhs.m_End = e;
+  return const_reverse_iterator(begin());
 }
 
 
-bool pqxx::result::field::operator==(const field &rhs) const
+pqxx::result::const_iterator pqxx::result::begin() const throw ()
 {
-  if (is_null() != rhs.is_null()) return false;
-  // TODO: Verify null handling decision
-  const size_type s = size();
-  if (s != rhs.size()) return false;
-  const char *const l(c_str()), *const r(rhs.c_str());
-  for (size_type i = 0; i < s; ++i) if (l[i] != r[i]) return false;
-  return true;
+  return const_iterator(this, 0);
 }
 
 
@@ -136,7 +116,7 @@ void pqxx::result::swap(result &rhs) throw ()
 }
 
 
-const pqxx::result::tuple pqxx::result::at(pqxx::result::size_type i) const
+const pqxx::tuple pqxx::result::at(pqxx::result::size_type i) const
   throw (range_error)
 {
   if (i >= size()) throw range_error("Tuple number out of range");
@@ -144,10 +124,10 @@ const pqxx::result::tuple pqxx::result::at(pqxx::result::size_type i) const
 }
 
 
-void pqxx::result::ThrowSQLError(const PGSTD::string &Err,
+void pqxx::result::ThrowSQLError(
+	const PGSTD::string &Err,
 	const PGSTD::string &Query) const
 {
-#if defined(PQXX_HAVE_PQRESULTERRORFIELD)
   // Try to establish more precise error type, and throw corresponding exception
   const char *const code = PQresultErrorField(m_data, PG_DIAG_SQLSTATE);
   if (code) switch (code[0])
@@ -191,7 +171,8 @@ void pqxx::result::ThrowSQLError(const PGSTD::string &Err,
     {
     case '2':
       if (strcmp(code,"42501")==0) throw insufficient_privilege(Err, Query);
-      if (strcmp(code,"42601")==0) throw syntax_error(Err, Query);
+      if (strcmp(code,"42601")==0)
+        throw syntax_error(Err, Query, errorposition());
       if (strcmp(code,"42703")==0) throw undefined_column(Err, Query);
       if (strcmp(code,"42883")==0) throw undefined_function(Err, Query);
       if (strcmp(code,"42P01")==0) throw undefined_table(Err, Query);
@@ -214,7 +195,6 @@ void pqxx::result::ThrowSQLError(const PGSTD::string &Err,
     if (strcmp(code, "P0003")==0) throw plpgsql_too_many_rows(Err, Query);
     throw plpgsql_error(Err, Query);
   }
-#endif
   throw sql_error(Err, Query);
 }
 
@@ -284,24 +264,26 @@ pqxx::result::size_type pqxx::result::affected_rows() const
 }
 
 
-const char *pqxx::result::GetValue(pqxx::result::size_type Row,
-		                 pqxx::result::tuple::size_type Col) const
+const char *pqxx::result::GetValue(
+	pqxx::result::size_type Row,
+	pqxx::tuple::size_type Col) const
 {
   return PQgetvalue(m_data, int(Row), int(Col));
 }
 
 
-bool pqxx::result::GetIsNull(pqxx::result::size_type Row,
-		           pqxx::result::tuple::size_type Col) const
+bool pqxx::result::GetIsNull(
+	pqxx::result::size_type Row,
+	pqxx::tuple::size_type Col) const
 {
   return PQgetisnull(m_data, int(Row), int(Col)) != 0;
 }
 
-pqxx::result::field::size_type
-pqxx::result::GetLength(pqxx::result::size_type Row,
-                        pqxx::result::tuple::size_type Col) const throw ()
+pqxx::field::size_type pqxx::result::GetLength(
+	pqxx::result::size_type Row,
+        pqxx::tuple::size_type Col) const throw ()
 {
-  return size_type(PQgetlength(m_data, int(Row), int(Col)));
+  return field::size_type(PQgetlength(m_data, int(Row), int(Col)));
 }
 
 
@@ -316,7 +298,6 @@ pqxx::oid pqxx::result::column_type(tuple::size_type ColNum) const
 }
 
 
-#ifdef PQXX_HAVE_PQFTABLE
 pqxx::oid pqxx::result::column_table(tuple::size_type ColNum) const
 {
   const oid T = PQftable(m_data, int(ColNum));
@@ -330,14 +311,11 @@ pqxx::oid pqxx::result::column_table(tuple::size_type ColNum) const
 
   return T;
 }
-#endif
 
 
-#ifdef PQXX_HAVE_PQFTABLECOL
-pqxx::result::tuple::size_type
-pqxx::result::table_column(tuple::size_type ColNum) const
+pqxx::tuple::size_type pqxx::result::table_column(tuple::size_type ColNum) const
 {
-  const tuple::size_type n = size_type(PQftablecol(m_data, int(ColNum)));
+  const tuple::size_type n = tuple::size_type(PQftablecol(m_data, int(ColNum)));
   if (n) return n-1;
 
   // Failed.  Now find out why, so we can throw a sensible exception.
@@ -357,43 +335,20 @@ pqxx::result::table_column(tuple::size_type ColNum) const
   throw usage_error("Can't query origin of column " + to_string(ColNum) + ": "
 	"not derived from table column");
 }
-#endif
 
 int pqxx::result::errorposition() const throw ()
 {
   int pos = -1;
-#if defined(PQXX_HAVE_PQRESULTERRORFIELD)
   if (m_data)
   {
     const char *p = PQresultErrorField(m_data, PG_DIAG_STATEMENT_POSITION);
     if (p) from_string(p, pos);
   }
-#endif // PQXX_HAVE_PQRESULTERRORFIELD
   return pos;
 }
 
 
-// tuple
-
-pqxx::result::field pqxx::result::tuple::at(const char f[]) const
-{
-  return field(*this, m_Begin + column_number(f));
-}
-
-
-pqxx::result::field
-pqxx::result::tuple::at(pqxx::result::tuple::size_type i) const
-  throw (range_error)
-{
-  if (i >= size())
-    throw range_error("Invalid field number");
-
-  return operator[](i);
-}
-
-
-const char *
-pqxx::result::column_name(pqxx::result::tuple::size_type Number) const
+const char *pqxx::result::column_name(pqxx::tuple::size_type Number) const
 {
   const char *const N = PQfname(m_data, int(Number));
   if (!N)
@@ -403,102 +358,30 @@ pqxx::result::column_name(pqxx::result::tuple::size_type Number) const
 }
 
 
-pqxx::result::tuple::size_type pqxx::result::columns() const throw ()
+pqxx::tuple::size_type pqxx::result::columns() const throw ()
 {
-  return m_data ? size_type(PQnfields(m_data)) : 0;
+  return m_data ? tuple::size_type(PQnfields(m_data)) : 0;
 }
 
 
-pqxx::result::tuple::size_type
-pqxx::result::tuple::column_number(const char ColName[]) const
+// const_result_iterator
+
+pqxx::const_result_iterator pqxx::const_result_iterator::operator++(int)
 {
-  const size_type n = m_Home->column_number(ColName);
-  if (n >= m_End)
-    return result().column_number(ColName);
-  if (n >= m_Begin)
-    return n - m_Begin;
-
-  const char *const AdaptedColName = m_Home->column_name(n);
-  for (size_type i = m_Begin; i < m_End; ++i)
-    if (strcmp(AdaptedColName, m_Home->column_name(i)) == 0)
-      return i - m_Begin;
-
-  return result().column_number(ColName);
-}
-
-
-pqxx::result::tuple::size_type
-pqxx::result::column_number(const char ColName[]) const
-{
-  const int N = PQfnumber(m_data, ColName);
-  // TODO: Should this be an out_of_range?
-  if (N == -1)
-    throw argument_error("Unknown column name: '" + string(ColName) + "'");
-
-  return tuple::size_type(N);
-}
-
-
-pqxx::result::tuple
-pqxx::result::tuple::slice(size_type Begin, size_type End) const
-{
-  if (Begin > End || End > size())
-    throw range_error("Invalid field range");
-
-  tuple result(*this);
-  result.m_Begin = m_Begin + Begin;
-  result.m_End = m_Begin + End;
-  return result;
-}
-
-
-bool pqxx::result::tuple::empty() const throw ()
-{
-  return m_Begin == m_End;
-}
-
-
-// const_iterator
-
-pqxx::result::const_iterator
-pqxx::result::const_iterator::operator++(int)
-{
-  const_iterator old(*this);
+  const_result_iterator old(*this);
   m_Index++;
   return old;
 }
 
 
-pqxx::result::const_iterator
-pqxx::result::const_iterator::operator--(int)
+pqxx::const_result_iterator pqxx::const_result_iterator::operator--(int)
 {
-  const_iterator old(*this);
+  const_result_iterator old(*this);
   m_Index--;
   return old;
 }
 
 
-
-// const_fielditerator
-
-pqxx::result::const_fielditerator
-pqxx::result::const_fielditerator::operator++(int)
-{
-  const_fielditerator old(*this);
-  m_col++;
-  return old;
-}
-
-
-pqxx::result::const_fielditerator
-pqxx::result::const_fielditerator::operator--(int)
-{
-  const_fielditerator old(*this);
-  m_col--;
-  return old;
-}
-
-
 pqxx::result::const_iterator
 pqxx::result::const_reverse_iterator::base() const throw ()
 {
@@ -507,47 +390,19 @@ pqxx::result::const_reverse_iterator::base() const throw ()
 }
 
 
-pqxx::result::const_reverse_iterator
-pqxx::result::const_reverse_iterator::operator++(int)
+pqxx::const_reverse_result_iterator
+pqxx::const_reverse_result_iterator::operator++(int)
 {
-  const_reverse_iterator tmp(*this);
+  const_reverse_result_iterator tmp(*this);
   iterator_type::operator--();
   return tmp;
 }
 
 
-pqxx::result::const_reverse_iterator
-pqxx::result::const_reverse_iterator::operator--(int)
+pqxx::const_reverse_result_iterator
+pqxx::const_reverse_result_iterator::operator--(int)
 {
-  const_reverse_iterator tmp(*this);
+  const_reverse_result_iterator tmp(*this);
   iterator_type::operator++();
   return tmp;
 }
-
-
-pqxx::result::const_fielditerator
-pqxx::result::const_reverse_fielditerator::base() const throw ()
-{
-  iterator_type tmp(*this);
-  return ++tmp;
-}
-
-
-pqxx::result::const_reverse_fielditerator
-pqxx::result::const_reverse_fielditerator::operator++(int)
-{
-  const_reverse_fielditerator tmp(*this);
-  operator++();
-  return tmp;
-}
-
-
-pqxx::result::const_reverse_fielditerator
-pqxx::result::const_reverse_fielditerator::operator--(int)
-{
-  const_reverse_fielditerator tmp(*this);
-  operator--();
-  return tmp;
-}
-
-
