@@ -7,7 +7,7 @@
  *      implementation of the pqxx::subtransaction class.
  *   pqxx::transaction is a nested transaction, i.e. one within a transaction
  *
- * Copyright (c) 2005-2009, Jeroen T. Vermeulen <jtv@xs4all.nl>
+ * Copyright (c) 2005-2011, Jeroen T. Vermeulen <jtv@xs4all.nl>
  *
  * See COPYING for copyright license.  If you did not receive a file called
  * COPYING with this source code, please notify the distributor of this mistake,
@@ -28,16 +28,33 @@ using namespace PGSTD;
 using namespace pqxx::internal;
 
 
-pqxx::subtransaction::subtransaction(dbtransaction &T,
-    const PGSTD::string &Name) :
+pqxx::subtransaction::subtransaction(
+	dbtransaction &T,
+	const PGSTD::string &Name) :
   namedclass("subtransaction", T.conn().adorn_name(Name)),
   transactionfocus(T),
   dbtransaction(T.conn(), false),
   m_parent(T)
 {
-#if defined(PQXX_HAVE_PQSERVERVERSION)
   check_backendsupport();
-#endif
+}
+
+
+namespace
+{
+typedef pqxx::dbtransaction &dbtransaction_ref;
+}
+
+
+pqxx::subtransaction::subtransaction(
+	subtransaction &T,
+	const PGSTD::string &Name) :
+  namedclass("subtransaction", T.conn().adorn_name(Name)),
+  transactionfocus(dbtransaction_ref(T)),
+  dbtransaction(T.conn(), false),
+  m_parent(T)
+{
+  check_backendsupport();
 }
 
 
@@ -49,9 +66,6 @@ void pqxx::subtransaction::do_begin()
   }
   catch (const sql_error &)
   {
-#if !defined(PQXX_HAVE_PQSERVERVERSION)
-    check_backendsupport();
-#endif
     throw;
   }
 }
