@@ -8,7 +8,7 @@
  *   pqxx::transaction_base defines the interface for any abstract class that
  *   represents a database transaction
  *
- * Copyright (c) 2001-2009, Jeroen T. Vermeulen <jtv@xs4all.nl>
+ * Copyright (c) 2001-2011, Jeroen T. Vermeulen <jtv@xs4all.nl>
  *
  * See COPYING for copyright license.  If you did not receive a file called
  * COPYING with this source code, please notify the distributor of this mistake,
@@ -21,9 +21,12 @@
 #include <cstring>
 #include <stdexcept>
 
+#ifdef PQXX_QUIET_DESTRUCTORS
+#include "pqxx/errorhandler"
+#endif
+
 #include "pqxx/connection_base"
 #include "pqxx/result"
-#include "pqxx/tablestream"
 #include "pqxx/transaction_base"
 
 #include "pqxx/internal/gates/connection-transaction.hxx"
@@ -48,12 +51,14 @@ pqxx::result pqxx::internal::parameterized_invocation::exec()
 {
   scoped_array<const char *> values;
   scoped_array<int> lengths;
-  const int elements = marshall(values, lengths);
+  scoped_array<int> binaries;
+  const int elements = marshall(values, lengths, binaries);
 
   return gate::connection_parameterized_invocation(m_home).parameterized_exec(
 	m_query,
 	values.get(),
 	lengths.get(),
+	binaries.get(),
 	elements);
 }
 
@@ -79,7 +84,7 @@ pqxx::transaction_base::transaction_base(connection_base &C, bool direct) :
 pqxx::transaction_base::~transaction_base()
 {
 #ifdef PQXX_QUIET_DESTRUCTORS
-  disable_noticer Quiet(m_Conn);
+  quiet_errorhandler quiet(m_Conn);
 #endif
   try
   {
@@ -223,7 +228,14 @@ void pqxx::transaction_base::abort()
 string pqxx::transaction_base::esc_raw(const PGSTD::string &str) const
 {
   const unsigned char *p = reinterpret_cast<const unsigned char *>(str.c_str());
-  return m_Conn.esc_raw(p, str.size());
+  return conn().esc_raw(p, str.size());
+}
+
+
+string pqxx::transaction_base::quote_raw(const PGSTD::string &str) const
+{
+  const unsigned char *p = reinterpret_cast<const unsigned char *>(str.c_str());
+  return conn().quote_raw(p, str.size());
 }
 
 
@@ -279,13 +291,11 @@ pqxx::result pqxx::transaction_base::exec(const PGSTD::string &Query,
 }
 
 
-#ifdef PQXX_HAVE_PQEXECPARAMS
 pqxx::internal::parameterized_invocation
 pqxx::transaction_base::parameterized(const PGSTD::string &query)
 {
   return internal::parameterized_invocation(conn(), query);
 }
-#endif // PQXX_HAVE_PQEXECPARAMS
 
 
 pqxx::prepare::invocation
