@@ -117,8 +117,9 @@ void Test(connection_base &C, bool ExplicitAbort)
 
 void test_abort(transaction_base &t)
 {
+  test::create_pqxxevents(t);
   connection_base &c(t.conn());
-  t.abort();
+  t.commit();
   Test(c, true);
   Test(c, false);
 }
