@@ -80,6 +80,12 @@ void test_013(transaction_base &T)
   connection_base &C(T.conn());
   T.abort();
 
+  {
+    work T2(C);
+    test::create_pqxxevents(T2);
+    T2.commit();
+  }
+
   const string Table = "pqxxevents";
 
   pair<int,int> Before;
@@ -90,7 +96,7 @@ void test_013(transaction_base &T)
 	"Already have event for " + to_string(BoringYear) + "--can't test.");
 
   const FailedInsert DoomedTransaction(Table);
-  disable_noticer d(C);
+  quiet_errorhandler d(C);
   PQXX_CHECK_THROWS(
 	C.perform(DoomedTransaction),
 	deliberate_error,
