@@ -31,10 +31,10 @@ void test_016(transaction_base &T)
   PQXX_CHECK_EQUAL(
 	c->size(),
 	R.back().size(),
-	"Size mismatch between tuple iterator and back().");
+	"Size mismatch between pqxx::tuple iterator and back().");
 
   const string nullstr;
-  for (result::tuple::size_type i = 0; i < c->size(); ++i)
+  for (pqxx::tuple::size_type i = 0; i < c->size(); ++i)
     PQXX_CHECK_EQUAL(
 	c[i].as(nullstr),
 	R.back()[i].as(nullstr),
