@@ -34,8 +34,9 @@ public:
 };
 
 
-void test_017(connection_base &C, transaction_base &T)
+void test_017(transaction_base &T)
 {
+  connection_base &C(T.conn());
   T.abort();
   C.perform(ReadTables());
 }
