@@ -87,6 +87,12 @@ void test_018(transaction_base &T)
   connection_base &C(T.conn());
   T.abort();
 
+  {
+    work T2(C);
+    test::create_pqxxevents(T2);
+    T2.commit();
+  }
+
   const string Table = "pqxxevents";
 
   pair<int,int> Before;
@@ -99,7 +105,7 @@ void test_018(transaction_base &T)
   const FailedInsert DoomedTransaction(Table);
 
   {
-    disable_noticer d(C);
+    quiet_errorhandler d(C);
     PQXX_CHECK_THROWS(
 	C.perform(DoomedTransaction),
 	deliberate_error,
