@@ -55,7 +55,7 @@ void test_067(transaction_base &orgT)
     PQXX_CHECK_EQUAL(i->size(), R.columns(), "result::columns() is broken.");
 
     // Look for null fields
-    for (result::tuple::size_type f=0; f<i->size(); ++f)
+    for (pqxx::tuple::size_type f=0; f<i->size(); ++f)
     {
       NullFields[f] += i.at(f).is_null();
 
@@ -81,7 +81,7 @@ void test_067(transaction_base &orgT)
       // fields may be sorted.  Don't do anything fancy like trying to
       // detect numbers and comparing them as such, just compare them as
       // simple strings.
-      for (result::tuple::size_type f = 0; f < R.columns(); ++f)
+      for (pqxx::tuple::size_type f = 0; f < R.columns(); ++f)
       {
 	if (!j[f].is_null() && !i[f].is_null())
 	{
@@ -97,7 +97,7 @@ void test_067(transaction_base &orgT)
   cout << "Read " << to_string(R.size()) << " rows." << endl;
   cout << "Field \t Field Name\t Nulls\t Sorted" << endl;
 
-  for (result::tuple::size_type f = 0; f < R.columns(); ++f)
+  for (pqxx::tuple::size_type f = 0; f < R.columns(); ++f)
   {
     cout << to_string(f) << ":\t"
 	 << R.column_name(f) << '\t'
