@@ -12,29 +12,29 @@ using namespace pqxx;
 // name with unusal characters, and without polling.
 namespace
 {
-// Sample implementation of notification listener
-class TestListener : public notify_listener
+// Sample implementation of notification receiver.
+class TestListener : public notification_receiver
 {
-  bool m_Done;
+  bool m_done;
 
 public:
   explicit TestListener(connection_base &C, string Name) :
-    notify_listener(C, Name), m_Done(false)
+    notification_receiver(C, Name), m_done(false)
   {
   }
 
-  virtual void operator()(int be_pid)
+  virtual void operator()(const string &, int be_pid)
   {
-    m_Done = true;
+    m_done = true;
     PQXX_CHECK_EQUAL(
 	be_pid,
-	Conn().backendpid(),
+	conn().backendpid(),
 	"Got notification from wrong backend process.");
 
-    cout << "Received notification: " << name() << " pid=" << be_pid << endl;
+    cout << "Received notification: " << channel() << " pid=" << be_pid << endl;
   }
 
-  bool Done() const { return m_Done; }
+  bool done() const { return m_done; }
 };
 
 
@@ -76,10 +76,10 @@ void test_078(transaction_base &orgT)
   TestListener L(C, NotifName);
 
   cout << "Sending notification..." << endl;
-  C.perform(Notify(L.name()));
+  C.perform(Notify(L.channel()));
 
   int notifs = 0;
-  for (int i=0; (i < 20) && !L.Done(); ++i)
+  for (int i=0; (i < 20) && !L.done(); ++i)
   {
     PQXX_CHECK_EQUAL(notifs, 0, "Got unexpected notifications.");
     cout << ".";
@@ -87,7 +87,7 @@ void test_078(transaction_base &orgT)
   }
   cout << endl;
 
-  PQXX_CHECK(L.Done(), "No notification received.");
+  PQXX_CHECK(L.done(), "No notification received.");
   PQXX_CHECK_EQUAL(notifs, 1, "Got unexpected number of notifications.");
 }
 } // namespace
