@@ -26,7 +26,7 @@ void test_083(transaction_base &orgT)
   try
   {
     nontransaction Drop(C, "drop_" + Table);
-    disable_noticer d(C);
+    quiet_errorhandler d(C);
     Drop.exec("DROP TABLE " + Table);
   }
   catch (const exception &e)
@@ -35,7 +35,7 @@ void test_083(transaction_base &orgT)
   }
 
   work T(C, "test83");
-  T.exec("CREATE TABLE " + Table + "(num INTEGER)");
+  T.exec("CREATE TEMP TABLE " + Table + "(num INTEGER)");
 
   tablewriter W(T, Table);
   back_insert_iterator<tablewriter> b(W);
