@@ -29,20 +29,15 @@ void compare_esc(connection_base &c, transaction_base &t, const char str[])
 	"Oversized buffer affects esc().");
 }
 
+
 void test_esc(connection_base &c, transaction_base &t)
 {
   PQXX_CHECK_EQUAL(t.esc("", 0), "", "Empty string doesn't escape properly.");
   PQXX_CHECK_EQUAL(t.esc("'", 1), "''", "Single quote escaped incorrectly.");
   const char *const escstrings[] = {
     "x",
+    " ",
     "",
-    "'",
-    "''",
-    "'''",
-    "''''",
-    "\\",
-    "\\\\",
-    "\\'",
     NULL
   };
   for (size_t i=0; escstrings[i]; ++i) compare_esc(c, t, escstrings[i]);
@@ -61,6 +56,41 @@ void test_quote(connection_base &c, transaction_base &t)
   PQXX_CHECK_EQUAL(t.quote("x"),
 	c.quote("x"),
 	"Connection and transaction quote differently.");
+
+  const char *test_strings[] = {
+    "",
+    "x",
+    "\\",
+    "\\\\",
+    "'",
+    "''",
+    "\\'",
+    "\t",
+    "\n",
+    NULL
+  };
+
+  for (size_t i=0; test_strings[i]; ++i)
+  {
+    result r = t.exec("SELECT " + t.quote(test_strings[i]));
+    PQXX_CHECK_EQUAL(
+	r[0][0].as<string>(),
+	test_strings[i],
+	"Selecting quoted string does not come back equal.");
+  }
+}
+
+
+void test_quote_name(transaction_base &t)
+{
+  PQXX_CHECK_EQUAL(
+	"\"A b\"",
+	t.quote_name("A b"),
+	"Escaped identifier is not as expected.");
+  PQXX_CHECK_EQUAL(
+	string("A b"),
+	t.exec("SELECT 1 AS " + t.quote_name("A b")).column_name(0),
+	"Escaped identifier does not work in SQL.");
 }
 
 
@@ -68,6 +98,7 @@ void test_escaping(transaction_base &t)
 {
   test_esc(t.conn(), t);
   test_quote(t.conn(), t);
+  test_quote_name(t);
 }
 } // namespace
 
