@@ -11,7 +11,7 @@ void test_thread_safety_model(transaction_base &)
 {
   const thread_safety_model model = describe_thread_safety();
 
-  if (model.have_strerror_r &&
+  if (model.have_safe_strerror &&
       model.safe_libpq &&
       model.safe_query_cancel &&
       model.safe_result_copy &&
